<?php

class Bookshelf_Addon
{

	public $PLUGIN_VERSION;
	public $PLUGIN_DIR_URL;
	public $PLUGIN_DIR_PATH;

	// Singleton
	private static $instance = null;

	public static function get_instance()
	{
		if (null == self::$instance) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	protected function __construct()
	{
		$this->add_actions();
		register_activation_hook($this->my_plugin_basename(), array($this, 'activation_hook'));
	}

	public function activation_hook($network_wide)
	{
	}

	public function enqueue_scripts()
	{
		
	}

	protected function get_translation_array()
	{
		return array(
			'worker_src' => $this->my_plugin_url() . 'js/pdfjs/pdf.worker.min.js',
			'cmap_url' => $this->my_plugin_url() . 'js/pdfjs/cmaps/',
			'objectL10n' => array(
				'loading' => esc_html__('Loading...', 'bookshelf')

			)
		);
	}

	public function admin_link($links)
	{
		array_unshift($links, '<a href="' . get_admin_url() . 'options-general.php?page=book_shelves">Admin</a>');
		return $links;
	}

	protected function get_options_menuname()
	{
		return 'bookshelf_options';
	}

	protected function get_options_pagename()
	{
		return 'bookshelf_options';
	}

	public function admin_menu()
	{
	}

	public function admin_init()
	{
	}

	public function init()
	{

		add_shortcode('bookshelf', array($this, 'on_shortcode'));

		wp_register_style(
			'edit_shelf_style',
			$this->PLUGIN_DIR_URL . "css/edit-shelf.css",
			array(),
			$this->PLUGIN_VERSION
		);

		wp_register_script("real3d_flipbook_shelves", $this->PLUGIN_DIR_URL . "js/shelves.js", array('jquery'), $this->PLUGIN_VERSION);


		wp_register_script(
			'shelf-color-picker',
			$this->PLUGIN_DIR_URL . 'js/alpha-color-picker.js',
			array('jquery', 'wp-color-picker'),
			$this->PLUGIN_VERSION,
			true
		);

		wp_register_style(
			'shelf-color-picker-style',
			$this->PLUGIN_DIR_URL . 'css/alpha-color-picker.css',
			array('wp-color-picker'),
			$this->PLUGIN_VERSION
		);

		wp_register_script(
			"edit_shelf",
			$this->PLUGIN_DIR_URL . "js/edit-shelf.js",
			array('alpha-color-picker', 'jquery-ui-sortable', 'jquery-ui-resizable', 'jquery-ui-selectable', 'wp-color-picker'),
			$this->PLUGIN_VERSION
		);
	}

	public function plugins_loaded()
	{
		load_plugin_textdomain('bookshelf', false, dirname($this->my_plugin_basename()) . '/lang/');
	}

	protected function add_actions()
	{

		add_action('plugins_loaded', array($this, 'plugins_loaded'));

		add_action('init', array($this, 'init'));

		add_action('wp_enqueue_scripts', array($this, 'enqueue_scripts'), 5, 0);

		if (is_admin()) {

			add_action('admin_init', array($this, 'admin_init'), 5, 0);

			add_action('admin_menu', array($this, 'admin_menu'));

			add_filter('plugin_action_links_' . plugin_basename(__FILE__), array($this, 'admin_link'));
			add_action('wp_ajax_r3d_shelf_save', array($this, 'save_options'));
			add_action('wp_ajax_nopriv_r3d_shelf_save', array($this, 'save_options'));

			add_action('wp_ajax_r3d_import_shelves', array($this,  'ajax_import_shelves'));
			add_action('wp_ajax_nopriv_r3d_import_shelves', array($this,  'ajax_import_shelves'));

			add_action('real3d_flipbook_menu', array($this, 'real3d_flipbook_installed'));
		}
	}

	public function include_bookshelf_admin()
	{

		include_once(plugin_dir_path(__FILE__) . 'admin.php');
	}

	public function on_shortcode($atts, $content = null)
	{

		$args = shortcode_atts(
			array(
				'id' => '-1',
				'category' => '-1',
				'width' => '-1',
				'height' => '-1',
				'viewmode' => '-1',
				'image' => '-1',
			),
			$atts
		);

		$id = $atts['id'];
		$viewMode = '';
		if (isset($atts['viewmode']))
			$viewMode = $atts['viewmode'];

		$shelf = get_option('shelf_' . $id);
		$image = $shelf['image'];
		$covers = $shelf['covers'];
		$coverMarginH = $shelf['coverMarginH'];
		$coverShadow = $shelf['coverShadow'];
		$coversAlign = $shelf['coversAlign'];

		$shelfImageMarginTop = $shelf['shelfImageMarginTop'];
		$shelfBg = $shelf['shelfBg'];
		$shelfPadding = $shelf['shelfPadding'];

		$coverWidth = isset($shelf['coverWidth']) ? $shelf['coverWidth'] : "20";
		$shelfWidth = isset($shelf['shelfWidth']) ? $shelf['shelfWidth'] : "836";
		$category = isset($shelf['r3d_category']) ? $shelf['r3d_category'] : "";

		$output = '';

		if ($category) {
			$args = array(
				'post_type' => 'r3d',
				'posts_per_page' => -1,
				'tax_query' => array(
					array(
						'taxonomy' => 'r3d_category',
						'field' => 'term_id',
						'terms' => $category
					)
				)
			);

			$my_query = new WP_Query($args);

			$covers = array();


			while ($my_query->have_posts()) {

				$my_query->the_post();
				$post_id = get_the_ID();
				$flipbook_id = get_post_meta($post_id, 'flipbook_id', true);

				array_push($covers, array(
					'id' => $flipbook_id
				));

				wp_reset_postdata();
			}
		}

		$coversPerRow = floor(100 / (float)$coverWidth);
		$currentCoverCount = 0;
		$rowStarted = false;

		foreach ($covers as $cover) {

			if (!$rowStarted)
				$output .= '<div class="book-shelf" style="background-color:' . $shelfBg . ';padding:' . $shelfPadding . 'px; max-width:' . $shelfWidth . 'px!important;"><div class="book-shelf-covers" style="justify-content:' . $coversAlign . '; gap: ' . $coverMarginH . 'px;">';
			$rowStarted = true;

			$shortcodeText = '[real3dflipbook id="' . $cover['id'] . '" mode="lightbox"';

			if ($viewMode == "3d" || $viewMode == "2d" || $viewMode == "webgl" || $viewMode == "swipe") {
				$shortcodeText .= ' viewmode="' . $viewMode . '"';
			}

			$shortcodeText .= ' thumbcss="; box-shadow:' . $coverShadow . ';" containercss=";width: calc(' . $coverWidth . '% - ' . $coverMarginH . 'px);"]';

			$output .= do_shortcode($shortcodeText);

			$currentCoverCount++;

			if ($currentCoverCount % $coversPerRow == 0 || $currentCoverCount == count($covers)) {
				$output .= '</div>';
				if ($image) {
					$output .= '<img class="book-shelf-img" style="z-index:0; height: auto; max-width: 100%; vertical-align: top; margin: 0 auto; transform:translateY(' . $shelfImageMarginTop . '%); " class="shelf_img" alt="Shelf Wood" src="' . $image . '">';
				}
				$output .= '</div>';
				$rowStarted = false;
			}
		}

		wp_enqueue_style(
			'edit_shelf_style'
		);

		return $output;
	}

	public function real3d_flipbook_installed()
	{

		add_submenu_page('real3d_flipbook_admin', 'Shelves', 'Shelves', 'publish_posts', 'book_shelves', array($this, 'include_bookshelf_admin'));
	}

	public function save_options()
	{

		$current_id = '';

		if (isset($_GET['shelfId'])) {

			$current_id = $_GET['shelfId'];
			$shelf = get_option("shelf_" . (string)$current_id);

			$new = array_merge($shelf, $_POST);

			update_option('shelf_' . (string)$current_id, $new);
		}

		wp_die();
	}

	public function ajax_import_shelves()
	{

		check_ajax_referer('r3d_nonce', 'security');

		$json = stripslashes($_POST['shelves']);

		$newShelves = json_decode($json, true);

		if ((string)$json != "" && is_array($newShelves)) {

			$shelf_ids = get_option('shelf_ids');

			foreach ($shelf_ids as $id) {
				delete_option('shelf_' . (string)$id);
			}

			$shelf_ids = array();

			foreach ($newShelves as $s) {
				$id = $s['id'];

				add_option('shelf_' . (string)$id, $s);
				array_push($shelf_ids, (string)$id);
			}

			update_option('shelf_ids', $shelf_ids);
		}

		wp_die();
	}

	protected function my_plugin_basename()
	{
		$basename = plugin_basename(__FILE__);
		if ('/' . $basename == __FILE__) {
			$basename = basename(dirname(__FILE__)) . '/' . basename(__FILE__);
		}
		return $basename;
	}

	protected function my_plugin_url()
	{
		$basename = plugin_basename(__FILE__);
		if ('/' . $basename == __FILE__) {
			return plugins_url() . '/' . basename(dirname(__FILE__)) . '/';
		}
		return plugin_dir_url(__FILE__);
	}
}
